@@ -420,7 +420,7 @@ QString Nepomuk2::Query::Query::toSparqlQuery( SparqlFlags sparqlFlags ) const
         else if( d->m_fileMode == FileQuery::QueryFolders )
             fileModeTerm = AndTerm( folderTerm, NegationTerm::negateTerm( fileTerm ) );
         else
-            fileModeTerm = OrTerm( fileTerm, folderTerm );
+            fileModeTerm = fileTerm;
         term = AndTerm( term, fileModeTerm, d->createFolderFilter() );
     }
 
